struct File
{
    enum FileClass
    {
        Allowed,
        Obsolete,
        Unknown
    } fclass;
    
    std::string infohash;
    
    std::string file;          // .torrent
    std::string name;          // contained file
    std::string path;          // path/.torrent
    int_fast64_t length;       // of contained file
    int_fast64_t torrentsize;
    unsigned numfiles;
    
    std::map<std::string /* peer_id */,
             Peer> peers;

    uint_fast64_t completed; // total completed downloads
    
public:
    File()
        : fclass(Unknown),
          infohash(),
          file(),
          name(),
          path(),
          length(),
          torrentsize(),
          numfiles(),
          peers(),
          completed() { }
    File(const File& f)
        : fclass(f.fclass),
          infohash(f.infohash),
          file(f.file),
          name(f.name),
          path(f.path),
          length(f.length),
          torrentsize(f.torrentsize),
          numfiles(f.numfiles),
          peers(f.peers),
          completed(f.completed) { }
    File& operator= (const File& f)
    {
        fclass = (f.fclass);
        infohash = (f.infohash);
        file = (f.file);
        name = (f.name);
        path = (f.path);
        length = (f.length);
        torrentsize = (f.torrentsize);
        numfiles = (f.numfiles);
        peers = (f.peers);
        completed = (f.completed);
        
        return *this;
    }
    
    void CleanupPeers()
    {
        static int cleanup_counter = 0;
        // TODO: remove peers who haven't talked for a long time
        //       and keep the number of them low anyway.
        if(!cleanup_counter)
        {
            const int max_age = timeout_downloaders_interval; // 1 hour
            
            cleanup_counter = 100;
            time_t now = time(NULL);
            for(std::map<std::string, Peer>::iterator
                i=peers.begin(), j(i); i!=peers.end(); i=j)
            {
                j=i; ++j;
                if(now >= i->second.last_seen + max_age)
                    peers.erase(i);
            }
        }
    }
};
